package MojoMojo::Controller::Journal;

use strict;
use base 'Catalyst::Controller';


=head1 NAME

MojoMojo::Controller::Journal - Journaling for MojoMojo?

=head1 SYNOPSIS

    Very simple to use

=head1 DESCRIPTION

Would preferably get some sort of integration with MTOS and Angerwhale to allow embedding blogs at given 
node points here.

=head1 AUTHOR

Marcus Ramberg <mramberg@cpan.org>


=head1 LICENSE

This library is free software . You can redistribute it and/or modify it under
the same terms as perl itself.

=cut

1;
