h1#pageName, .newWikiWord a, a.existingWikiWord, .newWikiWord a:hover, #TextileHelp h3 { 
    color: #008B26; 
}

.leftHandSide {
  float: right;
  width: 147px;
  margin-left: 10px;
  padding-left: 20px;
  border-left: 1px dotted #ccc;
}
.leftHandSide p {
  font-size: 10px;
}
